unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, Printers,
  StdCtrls, Grids;

type
  TForm1 = class(TForm)
    Grid: TStringGrid;
    btnPrint: TButton;
    procedure FormCreate(Sender: TObject);
    
    procedure btnPrintClick(Sender: TObject);
  private
    procedure PrintGrid(const Grid: TStringGrid; const Title: string; Orientation: TPrinterOrientation);
    procedure GetZoneImpressionInPixels(PrinterHandle : HDC; var Height , Width : Integer);
  public

  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

//====================================================================================================
procedure TForm1.FormCreate(Sender: TObject);
var
  iCol, iRow : Integer;
begin
  for iCol := 0 to Grid.ColCount-1 do begin
    for iRow := 0 to Grid.RowCount-1 do begin
      Grid.Cells[iCol, iRow] := IntToStr(iCol*iRow);
    end;
  end;
end;

//====================================================================================================
procedure TForm1.btnPrintClick(Sender: TObject);
begin
  PrintGrid(Grid, 'Test d''impression', poPortrait);
end;

//====================================================================================================
procedure TForm1.PrintGrid(const Grid: TStringGrid; const Title: string; Orientation: TPrinterOrientation);
var
  iLOGPIXELSX, iRow, iCol, iWidth, iHeight, X, Y, iPage : Integer;
  R : TRect;
  sz : String;
  bDoCenter : Boolean;

  procedure PrintTitle;
  var
    iTextW : Integer;
    szPage : String;
  begin
    // Titre
    Printer.Canvas.Pen.Color  := 0;
    Printer.Canvas.Font.Name  := 'Times New Roman';
    Printer.Canvas.Font.Size  := 12;
    Printer.Canvas.Font.Style := [fsBold];
    iTextW := Printer.Canvas.TextWidth(Title);
    Printer.Canvas.TextOut((iWidth-iTextW) div 2, 100, Title);

    // N de page
    Printer.Canvas.Font.Style := [];
    szPage := '- '+IntToStr(iPage)+' -';
    iTextW := Printer.Canvas.TextWidth(szPage);
    Printer.Canvas.TextOut((iWidth-iTextW) div 2, iHeight-90, szPage);

    // Utilise la police de la grille (Pour l'impression des cellules)
    Printer.Canvas.Font.Name  := Grid.Font.Name;
    Printer.Canvas.Font.Size  := Grid.Font.Size;
    Printer.Canvas.Font.Style := Grid.Font.Style;
  end;

  function GetValue(const Value : Integer) : Integer;
  begin
    Result := MulDiv(Value, iLOGPIXELSX, Self.PixelsPerInch);
  end;

begin
  GetZoneImpressionInPixels(Printer.Handle, iHeight, iWidth);

  // Rcupre la rsolution de l'cran
  iLOGPIXELSX := GetDeviceCaps(Printer.Handle, LOGPIXELSX);
  iPage := 1;

  Printer.Title := Title;
  Printer.BeginDoc;

  // Titre
  PrintTitle;

  // Initialise le rectangle
  FillChar(R, SizeOf(TRect), #0);
  R.Bottom := 300;

  // Parcours toutes les lignes
  for iRow := 0 to Grid.RowCount-1 do begin
    R.Right := 0;
    R.Top := R.Bottom;
    R.Bottom := R.Top + GetValue(Grid.RowHeights[iRow]);

    // Pour l'impression des titres de colonne
    bDoCenter := (iRow=0) and (Grid.FixedRows>0);

    // Page suivante
    if (R.Bottom>iHeight) then begin
      inc(iPage);
      Printer.NewPage;
      PrintTitle;
      R.Top := 0;
      R.Bottom := R.Top + GetValue(Grid.RowHeights[iRow]);
    end;

    // Parcours toutes les colonnes
    for iCol := 0 to Grid.ColCount-1 do begin
      // Rectangle d'impression
      R.Left := R.Right;
      R.Right := R.Left + GetValue(Grid.ColWidths[iCol]);

      // Texte  imprimer
      sz := Grid.Cells[iCol, iRow];

      // Imprime le texte
      Y := R.Top + ((R.Bottom-R.Top-Printer.Canvas.TextHeight(sz)) div 2);
      if bDoCenter then X := R.Left + ((R.Right-R.Left-Printer.Canvas.TextWidth(sz)) div 2)
                   else X := R.Left+10;
      Printer.Canvas.TextRect(R, X, Y, sz);
      Printer.Canvas.MoveTo(R.Left, R.Top);
      Printer.Canvas.LineTo(R.Right, R.Top);
      Printer.Canvas.LineTo(R.Right, R.Bottom);
      Printer.Canvas.LineTo(R.Left, R.Bottom);
      Printer.Canvas.LineTo(R.Left, R.Top);
    end;
  end;
  Printer.EndDoc;
end;

//====================================================================================================
procedure TForm1.GetZoneImpressionInPixels(PrinterHandle: HDC; var Height, Width: Integer);
begin
  Height := GetDeviceCaps(PrinterHandle, VERTRES);
  Width := GetDeviceCaps(PrinterHandle, HORZRES);
end;

end.
